package core

import (
	"git.urbach.dev/cli/q/src/eval"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/token"
	"git.urbach.dev/cli/q/src/types"
)

// Evaluate evaluates an expression and returns a value.
func (f *Function) Evaluate(expr *expression.Expression) (eval.Value, error) {
	if expr.IsFolded {
		value := eval.Value{
			Kind:   eval.Number,
			Type:   types.AnyInt,
			Number: expr.Value,
		}

		return value, nil
	}

	if expr.IsLeaf() {
		return f.EvaluateLeaf(expr)
	}

	switch expr.Token.Kind {
	case token.Call:
		return f.EvaluateCall(expr)

	case token.Dot:
		return f.EvaluateDot(expr)

	case token.Array:
		return f.EvaluateArray(expr)
	}

	tmp := f.NewRegister()
	typ, err := f.ExpressionToRegister(expr, tmp)

	value := eval.Value{
		Kind:     eval.Register,
		Type:     typ,
		Register: tmp,
	}

	return value, err
}
