package core

import (
	"fmt"
	"math"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/eval"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/types"
)

// EvaluateArray evaluates a function call.
func (f *Function) EvaluateArray(expr *expression.Expression) (eval.Value, error) {
	name := expr.Children[0].Token.Text(f.File.Bytes)
	array := f.VariableByName(name)

	if array == nil {
		return eval.Value{}, errors.New(&errors.UnknownIdentifier{Name: name}, f.File, expr.Children[0].Token.Position)
	}

	defer f.UseVariable(array)

	memory := asm.Memory{
		Base:           array.Register,
		Offset:         0,
		OffsetRegister: math.MaxUint8,
		Length:         byte(1),
	}

	indexExpr := expr.Children[1]
	index, err := f.Evaluate(indexExpr)

	if err != nil {
		return eval.Value{}, err
	}

	if !types.Is(index.Type, types.AnyInt) {
		return eval.Value{}, errors.New(&errors.TypeMismatch{Encountered: index.Type.Name(), Expected: types.AnyInt.Name()}, f.File, indexExpr.Token.Position)
	}

	switch index.Kind {
	case eval.Number:
		memory.Offset = int8(index.Number)
	case eval.Register:
		memory.OffsetRegister = index.Register
		defer f.FreeRegister(index.Register)
	default:
		panic(fmt.Errorf("%s: not implemented: %d", f.UniqueName, index.Kind))
	}

	value := eval.Value{
		Kind:   eval.Memory,
		Type:   array.Type.(*types.Array).Of,
		Memory: memory,
	}

	return value, nil
}
