package scanner

import (
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/fs"
	"git.urbach.dev/cli/q/src/token"
)

// scanFunction scans a function.
func (s *Scanner) scanFunction(file *fs.File, tokens token.List, i int) (int, error) {
	function, i, err := scanFunctionSignature(file, tokens, i, token.BlockStart)

	if err != nil {
		return i, err
	}

	var (
		blockLevel = 0
		bodyStart  = -1
	)

	// Function definition
	for i < len(tokens) {
		if tokens[i].Kind == token.ReturnType {
			i++
			continue
		}

		if tokens[i].Kind == token.BlockStart {
			blockLevel++
			i++

			if blockLevel == 1 {
				bodyStart = i
			}

			continue
		}

		if tokens[i].Kind == token.BlockEnd {
			blockLevel--

			if blockLevel < 0 {
				return i, errors.New(errors.MissingBlockStart, file, tokens[i].Position)
			}

			if blockLevel == 0 {
				break
			}

			i++
			continue
		}

		if tokens[i].Kind == token.Invalid {
			return i, errors.New(&errors.InvalidCharacter{Character: tokens[i].Text(file.Bytes)}, file, tokens[i].Position)
		}

		if tokens[i].Kind == token.EOF {
			if blockLevel > 0 {
				return i, errors.New(errors.MissingBlockEnd, file, tokens[i].Position)
			}

			if bodyStart == -1 {
				return i, errors.New(errors.ExpectedFunctionDefinition, file, tokens[i].Position)
			}

			return i, nil
		}

		if blockLevel > 0 {
			i++
			continue
		}

		return i, errors.New(errors.ExpectedFunctionDefinition, file, tokens[i].Position)
	}

	function.Body = tokens[bodyStart:i]
	s.functions <- function
	i++
	return i, nil
}
